;;; guile-irrlicht --- FFI bindings for Irrlicht Engine
;;; Copyright (C) 2020 Javier Sancho <jsf@jsancho.org>
;;;
;;; This file is part of guile-irrlicht.
;;;
;;; Guile-irrlicht is free software; you can redistribute it and/or modify
;;; it under the terms of the GNU Lesser General Public License as
;;; published by the Free Software Foundation; either version 3 of the
;;; License, or (at your option) any later version.
;;;
;;; Guile-irrlicht is distributed in the hope that it will be useful, but
;;; WITHOUT ANY WARRANTY; without even the implied warranty of
;;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
;;; General Public License for more details.
;;;
;;; You should have received a copy of the GNU Lesser General Public
;;; License along with guile-irrlicht.  If not, see
;;; <http://www.gnu.org/licenses/>.


(define-module (irrlicht device)
  #:use-module (oop goops)
  #:use-module (irrlicht base)
  #:use-module (irrlicht foreign)
  #:use-module (irrlicht irr)
  #:use-module (irrlicht gui)
  #:use-module (irrlicht scene)
  #:use-module (irrlicht video))


;; IrrlichtDevice
(define-class <irrlicht-device> (<reference-counted>)
  (irr-class #:init-value "IrrlichtDevice" #:getter irr-class))

(define* (create-device #:key
                        (device-type 'software)
                        (window-size '(640 480))
                        (bits 16)
                        (fullscreen #f)
                        (stencilbuffer #f)
                        (vsync #f)
                        (receiver (make <event-receiver>)))
  (if (not (is-a? receiver <event-receiver>))
      (error
       "In procedure create-device: Wrong type argument (expecting <event-receiver>):"
       receiver))

  (make <irrlicht-device>
    #:irr-pointer
    ((get-irrlicht-proc "createDevice")
     device-type
     window-size
     bits
     fullscreen
     stencilbuffer
     vsync
     (irr-pointer receiver))))

(define-method (get-gui-environment (device <irrlicht-device>))
  (make <gui-environment>
    #:irr-pointer ((get-irrlicht-proc "getGUIEnvironment" device) (irr-pointer device))))

(define-method (get-scene-manager (device <irrlicht-device>))
  (make <scene-manager>
    #:irr-pointer ((get-irrlicht-proc "getSceneManager" device) (irr-pointer device))))

(define-method (get-video-driver (device <irrlicht-device>))
  (make <video-driver>
    #:irr-pointer ((get-irrlicht-proc "getVideoDriver" device) (irr-pointer device))))

(define-method (set-window-caption! (device <irrlicht-device>) text)
  ((get-irrlicht-proc "setWindowCaption" device)
   (irr-pointer device) text))

(export create-device get-gui-environment get-scene-manager get-video-driver set-window-caption!)
