/* guile-irrlicht --- GNU Guile bindings for Irrlicht Engine

   Copyright (C) 2020 Javier Sancho <jsf@jsancho.org>

   This file is part of guile-irrlicht.

   guile-irrlicht is free software; you can redistribute it and/or modify
   it under the terms of the GNU Lesser General Public License as
   published by the Free Software Foundation; either version 3 of the
   License, or (at your option) any later version.

   guile-irrlicht is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   General Public License for more details.

   You should have received a copy of the GNU Lesser General Public
   License along with guile-irrlicht. If not, see
   <http://www.gnu.org/licenses/>.
*/

#include <irrlicht/irrlicht.h>
#include <libguile.h>
#include "gsubr.h"
#include "gui-environment.h"
#include "position2d.h"
#include "rect.h"
#include "wchar.h"


using namespace irr;


template <typename TParent>
SCM
IGUIEnvironment_addButton (SCM gui_environment,
                           SCM rectangle,
                           SCM parent,
                           SCM id,
                           SCM text,
                           SCM tooltiptext)
{
  gui::IGUIButton* button =
    ((gui::IGUIEnvironment*)scm_to_pointer (gui_environment))->
    addButton (scm_to_rect_s32 (rectangle),
               (TParent)scm_to_pointer (parent),
               scm_to_int32 (id),
               scm_to_wide_char_string (text),
               scm_to_wide_char_string (tooltiptext));
  return scm_from_pointer ((void*)button, NULL);
}


template <typename TParent>
SCM
IGUIEnvironment_addEditBox (SCM gui_environment,
                            SCM text,
                            SCM rectangle,
                            SCM border,
                            SCM parent,
                            SCM id)
{
  gui::IGUIEditBox* editbox =
    ((gui::IGUIEnvironment*)scm_to_pointer (gui_environment))->
    addEditBox (scm_to_wide_char_string (text),
                scm_to_rect_s32 (rectangle),
                scm_to_bool (border),
                (TParent)scm_to_pointer (parent),
                scm_to_int32 (id));
  return scm_from_pointer ((void*)editbox, NULL);
}


template <typename TParent>
SCM
IGUIEnvironment_addImage (SCM gui_environment,
                          SCM image,
                          SCM position,
                          SCM use_alpha_channel,
                          SCM parent,
                          SCM id,
                          SCM text)
{
  gui::IGUIEnvironment* guienv = (gui::IGUIEnvironment*)scm_to_pointer (gui_environment);
  gui::IGUIImage* new_image =
    guienv->addImage ((video::ITexture*)scm_to_pointer (image),
                      scm_to_position2d_s32 (position),
                      scm_to_bool (use_alpha_channel),
                      (TParent)scm_to_pointer (parent),
                      scm_to_int32 (id),
                      scm_to_wide_char_string (text));
  return scm_from_pointer ((void*) new_image, NULL);
}


template <typename TParent>
SCM
IGUIEnvironment_addListBox (SCM gui_environment,
                            SCM rectangle,
                            SCM parent,
                            SCM id,
                            SCM draw_background)
{
  gui::IGUIListBox* listbox =
    ((gui::IGUIEnvironment*)scm_to_pointer (gui_environment))->
    addListBox (scm_to_rect_s32 (rectangle),
                (TParent)scm_to_pointer (parent),
                scm_to_int32 (id),
                scm_to_bool (draw_background));
  return scm_from_pointer ((void*)listbox, NULL);
}


template <typename TParent>
SCM
IGUIEnvironment_addScrollBar (SCM gui_environment,
                              SCM horizontal,
                              SCM rectangle,
                              SCM parent,
                              SCM id)
{
  gui::IGUIScrollBar* scrollbar =
    ((gui::IGUIEnvironment*)scm_to_pointer (gui_environment))->
    addScrollBar (scm_to_bool (horizontal),
                  scm_to_rect_s32 (rectangle),
                  (TParent)scm_to_pointer (parent),
                  scm_to_int32 (id));
  return scm_from_pointer ((void*)scrollbar, NULL);
}


template <typename TParent>
SCM
IGUIEnvironment_addStaticText (SCM gui_environment,
                               SCM text,
                               SCM rectangle,
                               SCM border,
                               SCM word_wrap,
                               SCM parent,
                               SCM id,
                               SCM fill_background)
{
  gui::IGUIStaticText* static_text =
    ((gui::IGUIEnvironment*)scm_to_pointer (gui_environment))->
    addStaticText (scm_to_wide_char_string (text),
                   scm_to_rect_s32 (rectangle),
                   scm_to_bool (border),
                   scm_to_bool (word_wrap),
                   (TParent)scm_to_pointer (parent),
                   scm_to_int32 (id),
                   scm_to_bool (fill_background));
  return scm_from_pointer ((void*)static_text, NULL);
}


template <typename TParent>
SCM
IGUIEnvironment_addWindow (SCM gui_environment,
                           SCM rectangle,
                           SCM modal,
                           SCM text,
                           SCM parent,
                           SCM id)
{
  gui::IGUIWindow* window =
    ((gui::IGUIEnvironment*)scm_to_pointer (gui_environment))->
    addWindow (scm_to_rect_s32 (rectangle),
               scm_to_bool (modal),
               scm_to_wide_char_string (text),
               (TParent)scm_to_pointer (parent),
               scm_to_int32 (id));
  return scm_from_pointer ((void*)window, NULL);
}


SCM
IGUIEnvironment_drawAll (SCM gui_environment)
{
  ((gui::IGUIEnvironment*)scm_to_pointer (gui_environment))->drawAll ();
  return SCM_UNSPECIFIED;
}


SCM
IGUIEnvironment_getBuiltInFont (SCM gui_environment)
{
  gui::IGUIFont* font =
    ((gui::IGUIEnvironment*)scm_to_pointer (gui_environment))->getBuiltInFont ();
  return scm_from_pointer ((void*)font, NULL);
}


SCM
IGUIEnvironment_getFont (SCM gui_environment,
                         SCM filename)
{
  gui::IGUIFont* font =
    ((gui::IGUIEnvironment*)scm_to_pointer (gui_environment))->
    getFont (scm_to_utf8_stringn (filename, NULL));
  return scm_from_pointer ((void*)font, NULL);
}


SCM
IGUIEnvironment_getSkin (SCM gui_environment)
{
  gui::IGUISkin* skin =
    ((gui::IGUIEnvironment*)scm_to_pointer (gui_environment))->getSkin ();
  return scm_from_pointer ((void*)skin, NULL);
}


extern "C" {

  void
  init_gui_environment (void)
  {
    DEFINE_GSUBR ("IGUIEnvironment_addButton_IGUIElement", 6, 0, 0,
                  IGUIEnvironment_addButton<gui::IGUIElement*>);
    DEFINE_GSUBR ("IGUIEnvironment_addEditBox_IGUIElement", 6, 0, 0,
                  IGUIEnvironment_addEditBox<gui::IGUIElement*>);
    DEFINE_GSUBR ("IGUIEnvironment_addImage_IGUIElement", 7, 0, 0,
                  IGUIEnvironment_addImage<gui::IGUIElement*>);
    DEFINE_GSUBR ("IGUIEnvironment_addListBox_IGUIElement", 5, 0, 0,
                  IGUIEnvironment_addListBox<gui::IGUIElement*>);
    DEFINE_GSUBR ("IGUIEnvironment_addScrollBar_IGUIElement", 5, 0, 0,
                  IGUIEnvironment_addScrollBar<gui::IGUIElement*>);
    DEFINE_GSUBR ("IGUIEnvironment_addStaticText_IGUIElement", 8, 0, 0,
                  IGUIEnvironment_addStaticText<gui::IGUIElement*>);
    DEFINE_GSUBR ("IGUIEnvironment_addWindow_IGUIElement", 6, 0, 0,
                  IGUIEnvironment_addWindow<gui::IGUIElement*>);
    DEFINE_GSUBR ("IGUIEnvironment_drawAll", 1, 0, 0, IGUIEnvironment_drawAll);
    DEFINE_GSUBR ("IGUIEnvironment_getBuiltInFont", 1, 0, 0, IGUIEnvironment_getBuiltInFont);
    DEFINE_GSUBR ("IGUIEnvironment_getFont", 2, 0, 0, IGUIEnvironment_getFont);
    DEFINE_GSUBR ("IGUIEnvironment_getSkin", 1, 0, 0, IGUIEnvironment_getSkin);
  }

}
